import logging

from Kuyil.api.repository import Repository
from Kuyil.database import session_factory
from Kuyil.models.dataset_entity import DatasetEntity
from Kuyil.models.repository_entity import RepositoryEntity
from Kuyil.utils.date_time_util import Util


class Dataset:

    def __init__(self, dataset_class, instance, label, repository, comment=None,
                 meta_data=None, dataset_created_ts=Util.current_time_millis(),
                 dataset_modified_ts=Util.current_time_millis()
                 ):
        self.instance = instance
        self.dataset_class = dataset_class
        self.label = label
        self.repository = repository
        self.comment = comment
        self.meta_data = meta_data
        self.dataset_created_ts = dataset_created_ts
        self.dataset_modified_ts = dataset_modified_ts
        self.id = None
        self.logger = logging.getLogger(__name__)

    def get_write_lock_if_read_or_idle(self, pid):
        pass

    def get_write_lock_if_idle(self, pid):
        pass

    def get_write_lock_if_write_read_or_idle(self, pid):

        """
        check if there is no active or inactive version with same pid
        if so:
            return None
        else:
            create a new version.
            Persist it in mysql table.
            Mutate internal field - id
            Query repository_entity from repository table using repository_name
            Take the write lock by incrementing write_lock_ts
            Persist
            return write_lock_ts
        """

        if self.__get_dataset_count_with_pid(pid):
            self.logger.info("Dataset with this pid already exists. Returning None")
            return None

        dataset_entity = self.__convert_dataset_to_entity(self)
        dataset_entity = self.__persist_dataset(dataset_entity)
        id = dataset_entity.id
        if id:
            self.logger.debug("This is the id of the new dataset : " + str(dataset_entity.id))
            self.id = id
            self.logger.debug("Get write lock on Dataset and update it")
            dataset_entity.write_lock_id = pid
            dataset_entity.write_lock_ts = Util.lock_time_millis()
            dataset_entity.is_incomplete = True
            dataset_entity.is_valid = False
            self.__persist_dataset(dataset_entity)
            return dataset_entity.write_lock_ts
        else:
            self.logger.warn("An error occurred while persisting dataset")
            return None

    def __get_dataset_count_with_pid(self, pid):
        session = session_factory()
        return session.query(DatasetEntity). \
            filter(DatasetEntity.label == self.label). \
            filter(DatasetEntity.dataset_class == self.dataset_class). \
            filter(DatasetEntity.instance == self.instance). \
            filter(DatasetEntity.repository == self.repository). \
            filter(DatasetEntity.write_lock_id == pid).count()

    def maintain_write_lock(self, pid):

        """
        Mantain the write lock by incrementing the write_lock_ts value by 5 mins
        :param pid:
        :return: updated write_lock_ts

        k = Query for the dataset qualifier dataset_class, instance, label, pid
        if k:
            update the write_lock_ts to current_time + 5 mins
        else :
            :exception
        """
        entity = self.__get_dataset_with_pid(self, pid)
        current_time = Util.current_time_millis()
        if entity and entity.write_lock_ts >= current_time:
            entity.write_lock_ts = Util.lock_time_millis()
            entity.dataset_modified_ts = current_time
            self.__persist_dataset(entity)
            return entity.write_lock_ts
        else:
            self.logger.info("Cannot obtain write lock as it expired or does not exist.")
            return None

    def write_failed(self, pid, comment=None):

        """
        :param pid:
        :param comment:
        :return: True/False on basis of success or failure

         k = Query for the dataset qualifier dataset_class, instance, label, pid
         if k:
            release write lock by updating write_lock_ts to current_timestamp
            update the comment
         else :
            :exception

        """
        entity = self.__get_dataset_with_pid(self, pid)
        if entity and entity.write_lock_ts >= Util.current_time_millis():
            current_time = Util.current_time_millis()
            entity.write_lock_ts = current_time
            entity.comment = comment
            entity.write_lock_id = None
            entity.dataset_modified_ts = current_time
            self.__persist_dataset(entity)
            return True
        else:
            return False

    def write_success(self, pid, comment=None):

        """
        :param pid:
        :param comment:
        :return: True/False on basis of success or failure

                 k = Query for the dataset qualifier dataset_class, instance, label, pid
                 if k:
                    release write lock by updating write_lock_ts to current_timestamp
                    update is_incomplete to false
                    update is_valid to true
                    update pid to None  so as to resolve conflict
                    update the comment
                 else :
                    :exception
        """

        entity = self.__get_dataset_with_pid(self, pid)

        if entity and entity.write_lock_ts >= Util.current_time_millis():
            entity.is_incomplete = False
            entity.is_valid = True
            entity.write_lock_id = None
            current_time = Util.current_time_millis()
            entity.write_lock_ts = current_time
            entity.dataset_ready_ts = current_time
            entity.dataset_modified_ts = current_time
            entity.comment = comment
            self.__persist_dataset(entity)
            return True
        else:
            return False

    def __get_latest_active_version(self):

        session = session_factory()
        entity = session.query(DatasetEntity). \
            filter(DatasetEntity.label == self.label). \
            filter(DatasetEntity.dataset_class == self.dataset_class). \
            filter(DatasetEntity.instance == self.instance). \
            filter(DatasetEntity.repository == self.repository). \
            filter(DatasetEntity.is_incomplete.is_(False)). \
            filter(DatasetEntity.is_valid.is_(True)).order_by(DatasetEntity.version.desc()).first()

        if entity:
            return entity
        else:
            self.logger.warn("No active version exists")
            return None

    def __get_latest_version(self):
        session = session_factory()
        entity = session.query(DatasetEntity). \
            filter(DatasetEntity.label == self.label). \
            filter(DatasetEntity.dataset_class == self.dataset_class). \
            filter(DatasetEntity.instance == self.instance). \
            filter(DatasetEntity.repository == self.repository). \
            order_by(DatasetEntity.version.desc()).first()

        if entity:
            return entity
        else:
            self.logger.warn("Can not find any latest version.")
            return None

    def __get_dataset_with_pid(self, dataset, pid=None):
        session = session_factory()
        self.logger.debug("Inside get_dataset_with_pid")
        if not pid:
            entity = session.query(DatasetEntity). \
                filter(DatasetEntity.label == self.label). \
                filter(DatasetEntity.dataset_class == self.dataset_class). \
                filter(DatasetEntity.instance == self.instance). \
                filter(DatasetEntity.repository == self.repository). \
                filter(DatasetEntity.id == dataset.id).first()
            self.logger.debug(entity)
            return entity
        else:
            entity = session.query(DatasetEntity). \
                filter(DatasetEntity.label == self.label). \
                filter(DatasetEntity.dataset_class == self.dataset_class). \
                filter(DatasetEntity.instance == self.instance). \
                filter(DatasetEntity.repository == self.repository). \
                filter(DatasetEntity.write_lock_id == pid).first()
            self.logger.debug(entity)
            return entity

    def __get_dataset(self, dataset_entity):
        session = session_factory()
        entity = session.query(DatasetEntity). \
            filter(DatasetEntity.label == dataset_entity.label). \
            filter(DatasetEntity.dataset_class == dataset_entity.dataset_class). \
            filter(DatasetEntity.instance == dataset_entity.instance). \
            filter(DatasetEntity.repository == self.repository). \
            filter(DatasetEntity.version == dataset_entity.version).first()

        if entity:
            return entity
        else:
            return None

    def __get_repository(self, name):
        session = session_factory()
        repo = session.query(RepositoryEntity). \
            filter(RepositoryEntity.name == name).first()

        self.logger.info("Name of the repo : " + repo.name)
        return Repository.__convert_entity_to_repository(repo)

    def __persist_dataset(self, dataset_entity):
        session = session_factory()
        session.add(dataset_entity)
        self.logger.debug("Persist this entity : ")
        self.logger.debug(dataset_entity)
        session.commit()
        return self.__get_dataset(dataset_entity)

    # need to fix this for multiple concurrent clients.
    # as of now mysql contraint won't allow same version through multiple clients
    def __convert_dataset_to_entity(self, dataset):
        entity = self.__get_latest_version()
        version = 1
        if entity:
            version += entity.version

        return DatasetEntity(dataset.dataset_class, dataset.instance, dataset.label, dataset.repository,
                             dataset.dataset_created_ts, dataset.dataset_modified_ts, dataset.comment, dataset.meta_data, version)

    def __convert_entity_to_dataset(self, dataset_entity):
        dataset = Dataset(dataset_entity.dataset_class, dataset_entity.instance, dataset_entity.label,
                          dataset_entity.repository, dataset_entity.comment, dataset_entity.meta_data)
        dataset.id = dataset_entity.id
        return dataset

    def get_read_lock(self):

        entity = self.__get_latest_active_version()
        if entity:
            self.id = entity.id
            entity.read_lock_ts = Util.lock_time_millis()
            self.__persist_dataset(entity)
            return True
        else:
            return False

    def mantain_read_lock(self):
        """
        Query dataset by Id rather than getting latest active version.
        :return:
        """
        entity = self.__get_dataset_with_pid(self)
        if entity:
            entity.read_lock_ts = Util.lock_time_millis()
            self.__persist_dataset(entity)
            return True
        else:
            return False

    def is_write_locked(self):
        entity = self.__get_dataset_with_pid(self)
        if not entity:
            return True
        if entity.write_lock_ts > Util.current_time_millis():
            return True
        else:
            return False

    def is_read_locked(self):
        entity = self.__get_dataset_with_pid(self)
        if not entity:
            return True
        if entity.read_lock_ts > Util.current_time_millis():
            return True
        else:
            return False

    def get_resource_uri(self):
        repository = self.__get_repository(self.repository)
        repository.get_resource_uri(self.dataset_class, self.instance, self.id)

    def set_validity(self, validity):
        """
        :param validity:
        :return:

        Query by Id and set validity
        """

        pass

    def set_deleted(self, is_deleted):
        """

        :param is_deleted:
        :return:

        Query by id and set is_deleted
        """
        pass

    def read_finished(self, pid, comment=None):
        """

        :param pid:
        :param comment:
        :return:

        Query by id and set read_lock_ts to current time.
        """
        entity = self.__get_dataset_with_pid(self)
        if entity and entity.read_lock_ts >= Util.current_time_millis():
            entity.read_lock_ts = Util.current_time_millis()
            self.__persist_dataset(entity)
            return True
        else:
            return False

    def get_metadata(self):
        return self.meta_data

    def set_metadata(self, **kwargs):

        pass

    def set_metadata_key_value(self, key, value):

        pass

    def get_class(self):
        return self.dataset_class

    def get_instance(self):
        return self.instance

    def get_label(self):
        return self.label




import json

from Kuyil.api.dataset import Dataset
from Kuyil.api.repository import Repository
from Kuyil.database import database_init
from Kuyil.database import session_factory
from Kuyil.models.repository_entity import RepositoryEntity
import logging
from Kuyil.models.dataset_entity import DatasetEntity


logger = logging.getLogger(__name__)


def get_repository(name):
    session = session_factory()
    repo = session.query(RepositoryEntity). \
        filter(RepositoryEntity.name == name).first()

    logger.info("Name of the repo : " + repo.name)
    return __convert_entity_to_repository(repo)


def create_dataset(dataset_class, instance, label, repository, comment=None,
                   meta_data=None, only_create=False):
    dataset = Dataset(dataset_class, instance, label, repository, comment, meta_data)
    entity = __get_dataset_latest_active_version(dataset)

    if only_create:
        if entity:
            logger.info("Latest active version is idle. Return None")
            return None

        else:
            logger.info("Latest active version is not available. Returning new Dataset")
            return dataset

    else:
        # if active version is present then return it else return the new Dataset created above
        if entity:
            logger.info("Latest active version is idle. Return latest active version")
            return __convert_entity_to_dataset(entity)
        else:
            logger.info("Latest active version is not available. Return new Dataset")
            return dataset


def get_datasets(dataset_class, instance_regex, repository, label=None, all_versions=False):
    pass


def replace_dataset_in_label(dataset_class, instance, source_label, target_label,
                             sink_label, keep_taget_modified_ts=True):
    pass


def init(file_path='config.json'):
    logger.debug("config file path : " + file_path)
    with open(file_path) as json_data_file:
        data = json.load(json_data_file)
    logger.debug(data)
    # Initializing the DB connection.
    database_init(data)


def __get_dataset_latest_active_version(dataset):
    session = session_factory()
    entity = session.query(DatasetEntity). \
        filter(DatasetEntity.label == dataset.label). \
        filter(DatasetEntity.dataset_class == dataset.dataset_class). \
        filter(DatasetEntity.instance == dataset.instance). \
        filter(DatasetEntity.repository == dataset.repository). \
        filter(DatasetEntity.is_incomplete.is_(False)). \
        filter(DatasetEntity.is_valid.is_(True)).order_by(DatasetEntity.version.desc()).first()

    if entity:
        return entity
    else:
        logger.info("No active version exists.")
        return None


def __convert_entity_to_dataset(dataset_entity):
    dataset = Dataset(dataset_entity.dataset_class, dataset_entity.instance, dataset_entity.label,
                      dataset_entity.repository, dataset_entity.comment, dataset_entity.meta_data)
    dataset.id = dataset_entity.id
    return dataset


def __convert_entity_to_repository(repositoryEntity):
    return Repository(repositoryEntity.name, repositoryEntity.config)



from Kuyil.database import *
import Kuyil.api as Kuyil


def test_initial_url_value():
    assert url == "mysql://root:root@54.153.107.17:3306/kuyil"


def test_url_after_conf_load():
    Kuyil.init("config.json")
    from Kuyil.database import url
    assert url == "mysql://root:root@localhost:3306/db"


def test_url_after_test_conf_load():
    test_init()
    from Kuyil.database import url
    assert url == "sqlite:///foo.db"


def test_engine_after_test_conf_load():
    test_init()
    from Kuyil.database import engine
    assert str(engine.url) == "sqlite:///foo.db"




from sqlalchemy import Column, Integer, String, BigInteger, Boolean, NVARCHAR, \
    DateTime, UniqueConstraint, VARCHAR
from sqlalchemy.sql import func

from Kuyil.models.base import Base


class DatasetEntity(Base):

    __tablename__ = 'dataset'

    id = Column(Integer, primary_key=True)
    dataset_class = Column(String(500))
    instance = Column(String(500))
    label = Column(String(100))
    version = Column(BigInteger, nullable=False)
    created_ts = Column(DateTime, nullable=False, server_default=func.now())
    repository = Column(String(100), nullable=False)
    tags = Column(String(500), nullable=True)
    alt_uri = Column(VARCHAR(200), nullable=True)
    read_lock_ts = Column(BigInteger, nullable=True)
    write_lock_id = Column(String(100), nullable=True)
    write_lock_ts = Column(BigInteger, nullable=True)
    is_expired = Column(Boolean, nullable=True, default=False)
    is_valid = Column(Boolean, default=False)
    is_incomplete = Column(Boolean, default=True)
    is_marked_for_deletion = Column(Boolean, nullable=True, default=False)
    is_deleted = Column(Boolean, nullable=True, default=False)
    modified_ts = Column(DateTime, nullable=False, server_default=func.now(), onupdate=func.now())
    dataset_created_ts = Column(BigInteger, nullable=True)
    dataset_ready_ts = Column(BigInteger, nullable=True)
    dataset_modified_ts = Column(BigInteger, nullable=True)
    dataset_marked_for_deletion_ts = Column(DateTime, nullable=True)
    dataset_deleted_ts = Column(DateTime, nullable=True)
    dataset_expired_ts = Column(DateTime, nullable=True)
    dataset_restored_ts = Column(DateTime, nullable=True)
    comment = Column(NVARCHAR(200), nullable=True)
    meta_data = Column(NVARCHAR(200), nullable=True)
    bytes = Column(BigInteger, nullable=True)
    lines = Column(Integer, nullable=True)

    UniqueConstraint('dataset_class', 'instance', 'label', 'version', 'repository')

    UniqueConstraint('dataset_class', 'instance', 'label', 'repository', 'write_lock_id')

    def __repr__(self):
        return "<Dataset(class='%s', instance='%s', label='%s', id='%s', is_incomplete='%s', is_valid='%s')>" % \
               (self.dataset_class, self.instance, self.label, self.id, self.is_incomplete, self.is_valid)

    # include comment, meta_data.
    def __init__(self, Class, instance, label, repository, dataset_created_ts,
                 dataset_modified_ts, comment=None, meta_data=None, version=None):
        self.dataset_class = Class
        self.instance = instance
        self.label = label
        self.repository = repository
        self.dataset_modified_ts = dataset_modified_ts
        self.dataset_created_ts = dataset_created_ts
        self.comment = comment
        self.meta_data = meta_data

        self.version = version
        self.is_valid = False
        self.is_incomplete = True




import json
import logging

from Kuyil.models.repository_entity import RepositoryEntity
from Kuyil.repository_client.repository_client_helper import get_repository_client


class Repository:

    def __init__(self, name, config):
        self.name = name
        self.config = json.loads(config)
        self.client = get_repository_client(self)
        self.logger = logging.getLogger(__name__)

    def init(self):
        self.client = get_repository_client(self)

    # avoiding **kwargs in the arguments
    def create_uri(self, dataset_class, instance, id, overwrite=False):
        bucket = self.config.get("bucket", "Kuyil")
        prefix = self.config.get("prefix", "test") + "/" + str(dataset_class) + "/" \
                        + str(instance) + "/" + str(id) + "/"

        try:
            self.client.put_object(
                Bucket=bucket,
                Body='',
                Key=prefix
            )
            return "s3://" + bucket + "/" + prefix
        except Exception:
            return None


    def get_resource_uri(self, dataset_class, instance, id):
        """

        :param dataset_class:
        :param instance:
        :param id:
        :return: URL/None

        create URL from class, instance, id
        call does_uri_exists()
        if yes:
            return uri
        else:
            return None
        """
        bucket = self.config.get("bucket", "Kuyil")
        prefix = self.config.get("prefix", "test") + "/" + str(dataset_class) + "/" \
                 + str(instance) + "/" + str(id) + "/"

        uri = "s3://" + bucket + "/" + prefix
        return uri

    def does_uri_exists(self, dataset_class, instance, id):

        bucket = self.config.get("bucket", "Kuyil")
        prefix = self.config.get("prefix", "test") + "/" + str(dataset_class) + "/" \
                 + str(instance) + "/" + str(id) + "/"

        client = self.client
        response = client.list_objects_v2(
            Bucket=bucket,
            Prefix=prefix,
        )
        for obj in response.get('Contents', []):
            if prefix in obj['Key']:
                return True
        return False

    def delete_uri(self, dataset_class, instance, id):

        bucket = self.config.get("bucket", "Kuyil")
        prefix = self.config.get("prefix", "test") + "/" + str(dataset_class) + "/" \
                 + str(instance) + "/" + str(id) + "/"
        self.client.delete_object(Bucket=bucket, Key=prefix)

    @staticmethod
    def __convert_entity_to_repository(repositoryEntity):
        return Repository(repositoryEntity.name, repositoryEntity.config)

    @staticmethod
    def __convert_repository_to_entity(repository):
        return RepositoryEntity(repository.name, json.dumps(repository.config))


import time


class Util:

    @staticmethod
    def current_time_millis():
        return int(round(time.time() * 1000))

    @staticmethod
    def lock_time_millis():
        return int(round(time.time() * 1000)) + 5*60*1000



from os import path, remove
import logging
import logging.config
import json
import os
import sys

sys.path.insert(0, os.path.abspath('../../'))
sys.path.insert(0, os.path.abspath('../'))

# If applicable, delete the existing log file to generate a fresh log file during each execution
if path.isfile("python_logging.log"):
    remove("python_logging.log")

"""
with open("config.json", 'r') as logging_configuration_file:
    config_dict = json.load(logging_configuration_file)

logging.config.dictConfig(config_dict)
"""
# Log that the logger was configured
logger = logging.getLogger(__name__)
logger.info('Completed configuring logger()!')


from Kuyil.api.dataset import Dataset
import Kuyil.api as Kuyil
from Kuyil.database import session_factory
from Kuyil.database import test_init
from Kuyil.models.repository_entity import RepositoryEntity
from Kuyil.utils.date_time_util import Util
from sqlalchemy_utils import drop_database


def test_dataset_instance():
    dataset = Dataset("class", "instance", "label", "repository", None, None,
                       Util.current_time_millis(), Util.current_time_millis())
    assert dataset.dataset_class == "class"
    assert dataset.instance == "instance"
    assert dataset.label == "label"
    assert dataset.repository == "repository"


def test_dataset_created_time():
    time = Util.current_time_millis()
    dataset1 = Dataset("class1", "instance", "label", "repository", None, None,
                       Util.current_time_millis(), Util.current_time_millis())
    assert dataset1.dataset_created_ts <= Util.current_time_millis()
    assert dataset1.dataset_created_ts >= time


def test_dataset_modifies_time():
    time = Util.current_time_millis()
    dataset = Dataset("class1", "instance", "label", "repository", None, None,
                       Util.current_time_millis(), Util.current_time_millis())
    assert dataset.dataset_modified_ts <= Util.current_time_millis()
    assert dataset.dataset_modified_ts >= time


def test_create_dataset():
    test_init()
    session = session_factory()
    config = '{"aws_access_key":"","aws_secret_access_key":"","bucket":"scale-testing","prefix":"test-folder"}'
    repo = RepositoryEntity("s3-us-east", config)
    session.add(repo)
    session.commit()
    dataset = Kuyil.create_dataset("class", "instance", "label", "s3-us-east")
    assert dataset


def test_write_lock():
    test_init()
    from Kuyil.database import engine
    drop_database(engine.url)
    session = session_factory()
    config = '{"aws_access_key":"","aws_secret_access_key":"","bucket":"scale-testing","prefix":"test-folder"}'
    repo = RepositoryEntity("s3-us-east", config)
    session.add(repo)
    session.commit()
    dataset = Kuyil.create_dataset("class", "instance", "label", "s3-us-east")
    time = dataset.get_write_lock_if_write_read_or_idle(1)
    assert time
    assert time > Util.current_time_millis()
    assert (time - Util.lock_time_millis()) < 100


def test_get_write_lock_with_same_pid():
    test_init()
    session = session_factory()
    config = '{"aws_access_key":"","aws_secret_access_key":"","bucket":"scale-testing","prefix":"test-folder"}'
    repo = RepositoryEntity("s3-us-east", config)
    session.add(repo)
    session.commit()
    dataset = Kuyil.create_dataset("class", "instance", "label", "s3-us-east")
    time = dataset.get_write_lock_if_write_read_or_idle(2)
    time2 = dataset.get_write_lock_if_write_read_or_idle(2)
    assert time
    assert not time2


def test_write_success():
    test_init()
    from Kuyil.database import engine
    drop_database(engine.url)
    session = session_factory()
    config = '{"aws_access_key":"","aws_secret_access_key":"","bucket":"scale-testing","prefix":"test-folder"}'
    repo = RepositoryEntity("s3-us-east", config)
    session.add(repo)
    session.commit()
    dataset = Kuyil.create_dataset("class", "instance", "label", "s3-us-east")
    dataset.get_write_lock_if_write_read_or_idle(1)
    assert dataset.write_success(1, "Finished")


def test_write_failed():
    test_init()
    from Kuyil.database import engine
    drop_database(engine.url)
    session = session_factory()
    config = '{"aws_access_key":"","aws_secret_access_key":"","bucket":"scale-testing","prefix":"test-folder"}'
    repo = RepositoryEntity("s3-us-east", config)
    session.add(repo)
    session.commit()
    dataset = Kuyil.create_dataset("class", "instance", "label", "s3-us-east")
    dataset.get_write_lock_if_write_read_or_idle(1)
    assert dataset.write_failed(1, "Finished")




from setuptools import setup, find_packages

setup(name = 'Kuyil',
        version='0.6',
        license='BSD',
        url='https://github.com/4info/Kuyil',
        description='sprint 1 rc',
        author='rohitash',
        author_email='rohitash@sigmoid.com',
        packages=find_packages(),
        package_data={'': ['config.json']},
        data_files=[('.', ['config.json'])],
        include_package_data=True,

        install_requires=['SQLAlchemy==1.2.2',
                          'SQLAlchemy-Utils == 0.32.21',
                          'boto3 == 1.5.22',
                          'pytest-cov == 2.5.1',
                          'MySQL-python==1.2.5'
                          ],
)

__author__ = 'rohitash'


from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy_utils import database_exists, create_database
import logging

from models.base import Base

url = "mysql://root:root@54.153.107.17:3306/kuyil"
# engine = create_engine('mysql://root:root@localhost:3306/mydatabase')
engine = create_engine(url)
# use session_factory() to get a new Session
_SessionFactory = sessionmaker(bind=engine, autocommit=False, autoflush=False)

session = _SessionFactory()

logger = logging.getLogger(__name__)


def session_factory():
    if not database_exists(engine.url):
        logger.debug("created database")
        create_database(engine.url)
    Base.metadata.create_all(engine)
    global session
    if session:
        session.commit()
        return session
    else:
        session = _SessionFactory()
        session.commit()
        return session


def create_url(data):
    try:
        host = data['mysql']['host']
        port = data['mysql']['port']
        username = data['mysql']['user']
        password = data['mysql']['password']
        db = data['mysql']['db']
        mysql_url = 'mysql://' + username + ":" + password + "@" + host + ":" + port + "/" + db
        logger.info("mysql url : " + mysql_url)
        return mysql_url
    except KeyError as error:
        logger.error(error + " key is not found in the config")


def database_init(config):
    global url
    url = create_url(config)
    logger.info("mysql url is updated : " + url)
    global engine
    engine = create_engine(url)
    logger.debug("Engine is udpated")
    global _SessionFactory
    _SessionFactory = sessionmaker(bind=engine, autocommit=False, autoflush=True)
    logger.debug("session factory is updated")
    global session
    session = _SessionFactory()
    logger.debug("session is updated")


def test_init():
    global url
    url = "sqlite:///foo.db"
    logger.info("mysql url is updated : " + url)
    global engine
    engine = create_engine(url)
    logger.debug("Engine is udpated")
    global _SessionFactory
    _SessionFactory = sessionmaker(bind=engine, autocommit=False, autoflush=True)
    logger.debug("session factory is updated")
    global session
    session = _SessionFactory()
    logger.debug("session is updated")


import boto3


def get_repository_client(repository):
    if "s3" in repository.name:
        config = repository.config

        session = boto3.session.Session()

        s3_client = session.client(
            service_name='s3',
            aws_access_key_id=config.get('aws_access_key', ""),
            aws_secret_access_key=config.get('aws_secret_access_key', ""),
        )

        return s3_client

    else:
        None


